/*	Copyright (c) 1991 Richard F. Man.  All rights reserved.
 *	man@labrea.zko.dec.com, P.O. Box 6, North Chelmsford, MA 01863
 *	tabstop=4
 *
 *	$Log: execpriv.h%v $
 * Revision 1.1  1991/12/08  22:12:48  rfm
 * Initial revision
 *
 *
 */
#ifndef EXECPRIV_H
#define EXECPRIV_H

#define NUMBER_OF_CONNECTED_PORTS   8

typedef enum
    { INHIBIT_PORT = 100, SUPPRESS_PORT, CONNECT_PORT } CONNECTION_TYPE;

/*  generic queue.  Other types of queue add to this structure
 */
typedef struct queue
    {
    struct queue    *prev;
    struct queue    *succ;
    THREAD_ID       threadID;
    EXEC_ID         watchID;
    STRING_ATOM     nameAtom;
    } QUEUE, MSG_QUEUE;

/*	semaphore queue
 */
typedef struct squeue
    {
    struct queue    *prev;
    struct queue    *succ;
    THREAD_ID       threadID;
    EXEC_ID         watchID;
    STRING_ATOM     nameAtom;
    BOOL            detectOn;
    } SEMA_QUEUE;

/*	port queue
 */
typedef struct pqueue
    {
    struct queue    *prev;
    struct queue    *succ;
    THREAD_ID       threadID;
    EXEC_ID         watchID;
    STRING_ATOM     nameAtom;                        
    unsigned char   inhibitTimer;
    unsigned char   suppressTimer;
    PORT            *pPortValue;
    CONNECTION_TYPE destinationTypes[NUMBER_OF_CONNECTED_PORTS];
    struct pqueue   *destinations[NUMBER_OF_CONNECTED_PORTS];
    } PORT_QUEUE;

/*	timer queue
 */
typedef struct tqueue
    {
    struct queue    *prev;
    struct queue    *succ;
    THREAD_ID       threadID;
    EXEC_ID         watchID;
    STRING_ATOM     nameAtom;
    unsigned char   initialTimerInterval;
    unsigned char   currentTimerInterval;
    } TIMER_QUEUE;

/*	queue header
 */
typedef struct
    {
    QUEUE           *pFirst;
    QUEUE           *pLast;
    } QUEUE_HDR;

/*	non user callable functions
 */
void ExecCallReturn(void);
EXEC_ID ExecExecID(unsigned int prefix);
EXEC_ID ExecSwitch(void);

EXEC_ID MsgDeQueue(EXEC_ID *pWatchID);
void MsgEnQueue(THREAD_ID threadID, EXEC_ID watchID);

void QueDeleteQueue(QUEUE_HDR *pHdr, STRING_ATOM nameAtom);
void *QueDeQueue(QUEUE_HDR *pHdr);
BOOL QueEmpty(QUEUE_HDR *pHdr);
void QueEnQueue(QUEUE_HDR *pHdr, void *p);
void *QueFindItem(QUEUE_HDR *pHdr, STRING_ATOM nameAtom);
void QueForeachQueueItem(QUEUE_HDR *pHdr, void (*pFunc)(void *p));

void *SysGetStack(unsigned int size);
void SysInit();

#endif
