/*	Copyright (c) 1991 Richard F. Man.  All rights reserved.
 *	man@labrea.zko.dec.com, P.O. Box 6, North Chelmsford, MA 01863
 *	tabstop=4
 *
 *	$Log: standup.c%v $
 * Revision 1.1  1991/12/08  22:14:03  rfm
 * Initial revision
 *
 *
 */
#include <stdio.h>
#include "exec.h"
#include "simpwalk.h"

struct
	{
	char *name;
	int pos;
	int step;
	} servoTable[12] =
	{ {"alpha0"}, {"alpha1"}, {"alpha2"}, {"alpha3"}, {"alpha4"}, {"alpha5"},
	{"beta0"}, {"beta1"}, {"beta2"}, {"beta3"}, {"beta4"}, {"beta5"} };

#define alpha0Servo 0
#define alpha1Servo 1
#define alpha2Servo 2
#define alpha3Servo 3
#define alpha4Servo 4
#define alpha5Servo 5
#define beta0Servo  6
#define beta1Servo  7
#define beta2Servo  8
#define beta3Servo  9
#define beta4Servo  10
#define beta5Servo  11

IO_NAME alphaData[] = {
	{ "alpha0Pos.in", "alpha0Pos.out", alpha0Servo },
	{ "alpha1Pos.in", "alpha1Pos.out", alpha1Servo },
	{ "alpha2Pos.in", "alpha2Pos.out", alpha2Servo },
	{ "alpha3Pos.in", "alpha3Pos.out", alpha3Servo },
	{ "alpha4Pos.in", "alpha4Pos.out", alpha4Servo },
	{ "alpha5Pos.in", "alpha5Pos.out", alpha5Servo }
	};

IO_NAME betaData[] = {
	{ "beta0Pos.in", "beta0Pos.out", beta0Servo },
	{ "beta1Pos.in", "beta1Pos.out", beta1Servo },
	{ "beta2Pos.in", "beta2Pos.out", beta2Servo },
	{ "beta3Pos.in", "beta3Pos.out", beta3Servo },
	{ "beta4Pos.in", "beta4Pos.out", beta4Servo },
	{ "beta5Pos.in", "beta5Pos.out", beta5Servo }
	};

int servoWrite(int servo, POSITION pos)
	{

	if (pos == NEUTRAL) 
		{
		servoTable[servo].pos = 0;
		if (servo == alpha0Servo || servo == beta0Servo)
			SysPortb(servo == alpha0Servo ? 0 : 1, servoTable[servo].pos);
		}
	else if (servo <= alpha5Servo)
		{
		if (pos == ALPHA_INCR && servoTable[servo].pos < 127)
			{
			++servoTable[servo].pos;
			if (servo == alpha0Servo)
				SysPortb(0, servoTable[servo].pos);
			}
		else if (pos == ALPHA_DECR && -128 < servoTable[servo].pos)
			{
			--servoTable[servo].pos;
			if (servo == alpha0Servo)
				SysPortb(0, servoTable[servo].pos);
			}
		}
	else if (servoTable[servo].pos < pos)
		servoTable[servo].step = 1;
	else if (pos < servoTable[servo].pos)
		servoTable[servo].step = -1;
	return (servoTable[servo].pos);
	}

int servoReadWrite(int servo, PORT position)
	{
	int pos;

	pos = servoTable[servo].pos + servoTable[servo].step;
	if (pos < -128 || 127 < pos)
		servoTable[servo].step = 0;
	else
		{
		if (pos == position)
			servoTable[servo].step = 0;
		servoTable[servo].pos = pos;
		if (servo == beta0Servo)
			SysPortb(1, servoTable[servo].pos);
		}
	return (servoTable[servo].pos);
	}

/*	alpha position.  One per leg.
 */
void alphaPos(void)
	{
	IO_NAME *pData = &alphaData[ExecArg0];
	PORT portValue, position;
	STRING_ATOM output = StringToAtom(pData->outName);

	MsgOpenPort(output, &portValue);
	MsgWriteToPort(output, 
			servoWrite(pData->servoName, NEUTRAL));
	MsgWatchPort(StringToAtom(pData->inName), &position);
	while (MsgGetMsg())
		{
		MsgWriteToPort(output, 
				servoWrite(pData->servoName, position));
		}
	}

/*	beta position.  One per leg.
 */
void betaPos(void)
	{
	IO_NAME *pData = &betaData[ExecArg0];
	EXEC_ID id, timerID;
	PORT currentPos, portValue, position;
	STRING_ATOM output = StringToAtom(pData->outName);

	MsgOpenPort(output, &portValue);
	MsgWriteToPort(output, BETA_DOWN);

	servoWrite(pData->servoName, position = BETA_DOWN);
	while (servoReadWrite(pData->servoName, position) != BETA_DOWN)
		;
	MsgWatchPort(StringToAtom(pData->inName), &position);
	timerID = MsgWatchTimer(1);

	while (id = MsgGetMsg())
		{
		if (id == timerID)
			MsgWriteToPort(output, servoReadWrite(pData->servoName, position));
		else
			servoWrite(pData->servoName, position);
		}
	}

#ifndef HC11
void printServos(void)
	{
	int i;

	MsgWatchTimer(40);
	while (MsgGetMsg())
		{
#define s(i)	(servoTable[i].pos < 0 ? -1 : (servoTable[i].pos == 0 ? 0 : 1))
		printf("%d\t%d\t%d\t\t%d\t%d\t%d\n",
			s(5), s(4), s(3), s(11), s(10), s(9));
		printf("%d\t%d\t%d\t\t%d\t%d\t%d\n",
			s(2), s(1), s(0), s(8), s(7), s(6));
		printf("\n");
		}
	}
#endif
